@@ -1,14 +1,13 @@
 /**********************************************************************
- * $Id: TopologyPreservingSimplifier.h 3626 2012-05-09 18:02:09Z strk $
  *
  * GEOS - Geometry Engine Open Source
- * http://geos.refractions.net
+ * http://geos.osgeo.org
  *
  * Copyright (C) 2006 Refractions Research Inc.
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU Lesser General Licence as published
- * by the Free Software Foundation. 
+ * by the Free Software Foundation.
  * See the COPYING file for more information.
  *
  **********************************************************************
@@ -21,14 +20,13 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_SIMPLIFY_TOPOLOGYPRESERVINGSIMPLIFIER_H
-#define GEOS_SIMPLIFY_TOPOLOGYPRESERVINGSIMPLIFIER_H
+#pragma once
 
 #include <geos/export.h>
 #include <geos/geom/Geometry.h>
 #include <geos/simplify/TaggedLinesSimplifier.h>
-#include <memory> // for auto_ptr
-#include <map> 
+#include <memory> // for unique_ptr
+#include <map>
 
 #ifdef _MSC_VER
 #pragma warning(push)
@@ -39,53 +37,51 @@ namespace geos {
 namespace simplify { // geos::simplify
 
 /** \brief
- *
  * Simplifies a geometry, ensuring that
  * the result is a valid geometry having the
  * same dimension and number of components as the input.
  *
  * The simplification uses a maximum distance difference algorithm
  * similar to the one used in the Douglas-Peucker algorithm.
- * 
+ *
  * In particular, if the input is an areal geometry
  * ( Polygon or MultiPolygon )
- * 
+ *
  *  -  The result has the same number of shells and holes (rings) as the input,
  *     in the same order
  *  -  The result rings touch at <b>no more</b> than the number of touching point in the input
  *     (although they may touch at fewer points)
  *
  */
-class GEOS_DLL TopologyPreservingSimplifier
-{
+class GEOS_DLL TopologyPreservingSimplifier {
 
 public:
 
-	static std::auto_ptr<geom::Geometry> simplify(
-			const geom::Geometry* geom,
-			double tolerance);
+    static std::unique_ptr<geom::Geometry> simplify(
+        const geom::Geometry* geom,
+        double tolerance);
 
-	TopologyPreservingSimplifier(const geom::Geometry* geom);
+    TopologyPreservingSimplifier(const geom::Geometry* geom);
 
-	/** \brief
-	 * Sets the distance tolerance for the simplification.
-	 *
-	 * All vertices in the simplified geometry will be within this
-	 * distance of the original geometry.
-	 * The tolerance value must be non-negative.  A tolerance value
-	 * of zero is effectively a no-op.
-	 *
-	 * @param distanceTolerance the approximation tolerance to use
-	 */
-	void setDistanceTolerance(double tolerance);
+    /** \brief
+     * Sets the distance tolerance for the simplification.
+     *
+     * All vertices in the simplified geometry will be within this
+     * distance of the original geometry.
+     * The tolerance value must be non-negative.  A tolerance value
+     * of zero is effectively a no-op.
+     *
+     * @param tolerance the approximation tolerance to use
+     */
+    void setDistanceTolerance(double tolerance);
 
-	std::auto_ptr<geom::Geometry> getResultGeometry();
+    std::unique_ptr<geom::Geometry> getResultGeometry();
 
 private:
 
-	const geom::Geometry* inputGeom;
+    const geom::Geometry* inputGeom;
 
-	std::auto_ptr<TaggedLinesSimplifier> lineSimplifier;
+    std::unique_ptr<TaggedLinesSimplifier> lineSimplifier;
 
 };
 
@@ -95,15 +91,3 @@ private:
 #ifdef _MSC_VER
 #pragma warning(pop)
 #endif
-
-#endif // GEOS_SIMPLIFY_TOPOLOGYPRESERVINGSIMPLIFIER_H
-
-/**********************************************************************
- * $Log$
- * Revision 1.3  2006/04/13 16:04:10  strk
- * Made TopologyPreservingSimplifier implementation successfully build
- *
- * Revision 1.2  2006/04/13 14:25:17  strk
- * TopologyPreservingSimplifier initial port
- *
- **********************************************************************/
